/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */
package org.olat.course.assessment;

import java.io.File;
import java.util.List;

import org.olat.core.id.Identity;
import org.olat.course.assessment.handler.AssessmentConfig;
import org.olat.course.assessment.handler.AssessmentHandler;
import org.olat.course.nodes.CourseNode;
import org.olat.course.run.scoring.ScoreEvaluation;
import org.olat.course.run.userview.UserCourseEnvironment;
import org.olat.modules.assessment.Role;
import org.olat.modules.assessment.model.AssessmentRunStatus;

/**
 * 
 * Initial date: 20 Aug 2019<br>
 * 
 * @author uhensler, urs.hensler@frentix.com, http://www.frentix.com
 *
 */
public interface CourseAssessmentService {
	
	public AssessmentHandler getAssessmentHandler(CourseNode courseNode);
	
	public AssessmentConfig getAssessmentConfig(CourseNode node);

	public void updateUserScoreEvaluation(CourseNode courseNode, ScoreEvaluation scoreEvaluation,
			UserCourseEnvironment userCourseEnvironment, Identity coachingIdentity, boolean incrementAttempts, Role by);

	/**
	 * 
	 * @param courseNode
	 * @param userCourseEnvironment
	 * @return The completion of its current task before being committed and
	 *         official.
	 */
	public Double getUserCurrentRunCompletion(CourseNode courseNode, UserCourseEnvironment userCourseEnvironment);

	public void updateCurrentCompletion(CourseNode courseNode, UserCourseEnvironment userCourseEnvironment,
			Identity identity, Double currentCompletion, AssessmentRunStatus status, Role by);

	/**
	 * @param courseNode
	 * @param userCourseEnvironment
	 * @return the users attempts of the node
	 */
	public Integer getUserAttempts(CourseNode courseNode, UserCourseEnvironment userCourseEnvironment);

	/**
	 * Increments the users attempts for this node and this user.
	 * 
	 * @param courseNode
	 * @param userCourseEnvironment
	 * @param doneBy
	 */
	public void incrementUserAttempts(CourseNode courseNode, UserCourseEnvironment userCourseEnvironment, Role doneBy);

	/**
	 * Updates the users attempts for this node and this user.
	 * 
	 * @param courseNode
	 * @param userAttempts
	 * @param userCourseEnvironment
	 * @param coachingIdentity
	 * @param doneBy
	 */
	public void updateUserAttempts(CourseNode courseNode, Integer userAttempts,
			UserCourseEnvironment userCourseEnvironment, Identity coachingIdentity, Role doneBy);
	
	/**
	 * @param courseNode
	 * @param userCourseEnvironment
	 * @return the user comment for this user for this node, given by coach
	 */
	public String getUserComment(CourseNode courseNode, UserCourseEnvironment userCourseEnvironment);

	/**
	 * Updates the user comment for this node and this user. This comment is visible
	 * to the user.
	 * 
	 * @param courseNode
	 * @param userComment
	 * @param userCourseEnvironment
	 * @param coachingIdentity
	 */
	public void updatedUserComment(CourseNode courseNode, String userComment,
			UserCourseEnvironment userCourseEnvironment, Identity coachingIdentity);

	/**
	 * @param courseNode
	 * @param userCourseEnvironment
	 * @return The coach comment for this user for this node (not visible to user)
	 */
	public String getCoachComment(CourseNode courseNode, UserCourseEnvironment userCourseEnvironment);

	/**
	 * Updates the coach comment for this node and this user. This comment is not
	 * visible to the user.
	 * 
	 * @param courseNode
	 * @param coachComment
	 * @param userCourseEnvironment
	 */
	public void updateCoachComment(CourseNode courseNode, String coachComment,
			UserCourseEnvironment userCourseEnvironment);

	/**
	 * @param courseNode
	 * @param userCourseEnvironment The course environment of the assessed user.
	 * @return The list of assessment document associated with this user and course
	 *         element.
	 */
	public List<File> getIndividualAssessmentDocuments(CourseNode courseNode,
			UserCourseEnvironment userCourseEnvironment);

	/**
	 * Add a document for the assessed user, if allowed.
	 * 
	 * @param courseNode
	 * @param document              The document
	 * @param userCourseEnvironment The course environment of the assessed user
	 * @param coachingIdentity      The coach who upload the document
	 */
	public void addIndividualAssessmentDocument(CourseNode courseNode, File document, String filename,
			UserCourseEnvironment userCourseEnvironment, Identity coachingIdentity);

	/**
	 * Remove a document.
	 * 
	 * @param courseNode
	 * @param document              The document to remove
	 * @param userCourseEnvironment The course environment of the assessed user
	 * @param coachingIdentity      The coach who delete the document
	 */
	public void removeIndividualAssessmentDocument(CourseNode courseNode, File document,
			UserCourseEnvironment userCourseEnvironment, Identity coachingIdentity);
	
	/**
	 * 
	 * @param courseNode
	 * @param userCourseEnvironment The user course environment of the assessed
	 *                              identity
	 * @param identity              The identity which do the action
	 * @param doneBy                The role of the identity which do the action
	 */
	public void updateLastModifications(CourseNode courseNode, UserCourseEnvironment userCourseEnvironment,
			Identity identity, Role doneBy);

	/**
	 * @param courseNode
	 * @param userCourseEnvironment
	 * @return the users log of this node
	 */
	public String getUserLog(CourseNode courseNode, UserCourseEnvironment userCourseEnvironment);

}
